@@ -76,27 +76,19 @@ def read_time(bytes_, mili=0):
     minute  = (raw_time & 0b0000011111100000) >> 5
     second  = (raw_time & 0b0000000000011111) * 2
     second += mili // 100
-    micros  = (mili % 100) * 1000
+    micros  = (mili % 100) * 10000
     # we know theres an issue in some Linux based systems that make
     # 0xffffffff datetimes for some FileRecords (that don't seem to belong to
     # the files, some kind of temporary record) so we must check a few things:
-    if year < 1980:
-        year = 1980
-    if year >= 2107:
-        year = 2107
-    if day < 1:
-        day = 1
-    if month >= 12:
-        month = 12
-    elif month < 1:
-        month = 1
-    if hour >= 23:
-        hour = 23
-    if minute >= 59:
-        minute = 59
-    if second >= 59:
-        second = 59
+    year   = max(1980,  min(2107, year))  # clips the values to within range
+    month  = max(1,     min(12,  month))
+    day    = max(1,     day)
+    hour   = min(23,   hour)
+    minute = min(59, minute)
+    second = min(59, second)
     # that gives a few sanity checks and should catch that particular issue
+    #     note: we're still not checking the day/month correctly, we should
+    #           be using the calendar module for that
     dt = datetime.datetime(year, month, day, hour, minute, second, micros)
     return dt
 
@@ -145,7 +137,7 @@ class FileRecord:
     @name.setter
     def name(self, value):
         if isinstance(value, str):
-            value = bytes(value, "ascii")
+            value = bytes(value, "latin1")
         value = value.upper()  # not supporting long names for the moment
         name = value.translate(FILENAME_TRANS)
         name = name.replace(b"\x00", b"")
@@ -173,7 +165,7 @@ class FileRecord:
     @ext.setter
     def ext(self, value):
         if isinstance(value, str):
-            value = bytes(value, "ascii")
+            value = bytes(value, "latin1")
         value = value.upper()
         ext = value.translate(FILENAME_TRANS)
         ext = ext.replace(b"\x00", b"")
@@ -348,9 +340,29 @@ class FileRecord:
         value = value.replace(second=second, microsecond=0)
         self._modified = value
     
+    @property
+    def deleted(self):
+        """
+        Boolean for the FileRecord being marked as deleted.
+
+        Setting this property to True (or True-like values) re-writes the first
+        character the FileRecord's name attribute. Setting it to false checks
+        for a \\xe5 value on the first byte, and if it finds it, replaces it for
+        and underscore.
+        """
+        return self._name[0:1] == b"\xe5"
+    
+    @deleted.setter
+    def deleted(self, value):
+        if value:
+            self._name = b"\xe5" + self._name[1:]
+        else:
+            if self._name[0:1] == b"\xe5":
+                self._name = b"_" + self._name[1:]
+    
     def __repr__(self):
-        name = self.name.decode("ascii")
-        ext  = self.ext.decode("ascii")
+        name = self.name.decode("latin1")
+        ext  = self.ext.decode("latin1")
         if self.attributes["directory"]:
             showname = f"<DIR> {name}"
         else:
@@ -358,8 +370,8 @@ class FileRecord:
         return f"< DirectoryEntry: {showname}>"
     
     def __str__(self):
-        name = self.name.decode("ascii")
-        ext  = self.ext.decode("ascii")
+        name = self.name.decode("latin1")
+        ext  = self.ext.decode("latin1")
         if self.attributes["directory"]:
             showname = f"<DIR> {name}"
         else:
@@ -372,6 +384,7 @@ class FileRecord:
                f"    {'created':12}: {self.created}\n" 
                f"    {'last_access':12}: {self.last_access}\n" 
                f"    {'modified':12}: {self.modified}\n" 
+               f"    {'deleted':12}:{self.deleted:>12}\n" 
                f">"
         )
     
@@ -421,11 +434,22 @@ class Directory:
         self._parent     = parent
         self.files       = []
         if parent is None:
-            self.path = record.name.decode("ascii")
+            self.path = record.name.decode("latin1")
         else:
-            self.path = parent.path + PATH_SEP + record.name.decode("ascii")
+            self.path = parent.path + PATH_SEP + record.name.decode("latin1")
         self._parse()
     
+    def __repr__(self):
+        return f"< Directory {self.path}>"
+    
+    def __str__(self):
+        ret =  f"< Directory {self.path}\n"
+        ret += "  Files: [\n    "
+        ret += "\n    ".join([repr(f) for f in self.files])
+        ret += "\n  ]"
+        ret += "\n>"
+        return ret
+    
     def _parse(self):
         """
         Parses the directory cluster on disk, creating the FileRecords for every
@@ -489,9 +513,9 @@ class FileHandle:
         self._record     = record
         self._mode       = mode
         if parent is None:
-            self.path = (record.name + b"." + record.ext).decode("ascii")
+            self.path = (record.name + b"." + record.ext).decode("latin1")
         else:
-            self.path = parent.path + PATH_SEP + (record.name + b"." + record.ext).decode("ascii")
+            self.path = parent.path + PATH_SEP + (record.name + b"." + record.ext).decode("latin1")
         # this is all a lie (yet), we'll just open in "rb" mode
         # we will try to mimic the IOBase methods as close as possible, without
         # going crazy in it... right?
@@ -586,11 +610,12 @@ class FileHandle:
         c_idx = offset // bsize
         self._ccluster = record.cluster
         ncluster = self._filesystem.fat1[self._ccluster]
-        while (ncluster >= 0x0ffffff0) and c_idx > 0:
+        while (ncluster < 0x0ffffff0) and c_idx > 0:
             self._ccluster = ncluster
             c_idx -= 1
             ncluster = self._filesystem.fat1[self._ccluster]
-        if c_idx == 0:  # we got to the last cluster
+        if c_idx >= 0 and (ncluster >= 0x0ffffff0):  # we got to the last cluster
+            self._buffer     = bytearray(filesystem._read_cluster(self._ccluster))
             self._file_pos   = record.size
             self._buffer_pos = record.size % bsize
             return self._file_pos
@@ -638,6 +663,7 @@ class FAT32:
         self.number_of_fats      = nof
         self.sectors_per_fat     = spf
         self.root_cluster        = rc
+        self.root                = None  # we set its value later
         # and now some calculations...
         self.fat1_address = rs * bps
         self.fat2_address = (rs + spf) * bps
@@ -652,6 +678,7 @@ class FAT32:
         self.fat1 = []
         self.fat2 = []
         self._load_fats()
+        self._post_init()
     
     def __repr__(self):
         return f"< FAT32 @ {self._base_address} of {self.path}>"
@@ -706,3 +733,15 @@ class FAT32:
         length = nclusters * bps * spc
         self._handle.seek(pos)
         return self._handle.read(length)
+    
+    def _post_init(self):
+        """
+        Finishes setting up values in the filesystem. Everything that comes here
+        assumes the basic functionality of the filesystem is already working
+        and adds ease-of-use things.
+        """
+        root = Directory(self, None, cluster=self.root_cluster)
+        v_id = root.files[0]
+        if v_id.attributes["volume-id"]:
+            root.path = (v_id.name + v_id.ext).decode("ascii)")
+        self.root = root
